@@ -254,14 +254,7 @@ downdateQR <- function(Q1,Q2,R,col) {
   m = nrow(Q1)
   n = ncol(Q1)
 
-  a = .C("downdate1",
-    Q1=as.double(Q1),
-    R=as.double(R),
-    col=as.integer(col-1),
-    m=as.integer(m),
-    n=as.integer(n),
-    dup=FALSE,
-    package="selectiveInference")
+  a = downdate1_(as.matrix(Q1), R, col, m, n) # Rcpp call
 
   Q1 = matrix(a$Q1,nrow=m)
   R = matrix(a$R,nrow=n)
@@ -295,14 +288,14 @@ coef.lar <- function(object, s, mode=c("step","lambda"), ...) {
   if (mode=="step") {
     if (min(s)<0 || max(s)>k) stop(sprintf("s must be between 0 and %i",k))
     knots = 1:k
-    dec = FALSE
+    decreasing = FALSE
   } else {
     if (min(s)<min(lambda)) stop(sprintf("s must be >= %0.3f",min(lambda)))
     knots = lambda
-    dec = TRUE
+    decreasing = TRUE
   }
 
-  return(coef.interpolate(beta,s,knots,dec))
+  return(coef.interpolate(beta,s,knots,decreasing))
 }
 
 # Prediction function for lar
@@ -310,7 +303,10 @@ coef.lar <- function(object, s, mode=c("step","lambda"), ...) {
 predict.lar <- function(object, newx, s, mode=c("step","lambda"), ...) {
   beta = coef.lar(object,s,mode)
   if (missing(newx)) newx = scale(object$x,FALSE,1/object$sx)
-  else newx = scale(newx,object$bx,FALSE)
+  else {
+    newx = matrix(newx,ncol=ncol(object$x))
+    newx = scale(newx,object$bx,FALSE)
+  }
   return(newx %*% beta + object$by)
 }
 
@@ -371,20 +367,25 @@ larInf <- function(obj, sigma=NULL, alpha=0.1, k=NULL, type=c("active","all","ai
     for (j in 1:k) {
       if (verbose) cat(sprintf("Inference for variable %i ...\n",vars[j]))
 
-      Gj = G[1:nk[j],]
-      uj = rep(0,nk[j])
+      Aj = -G[1:nk[j],]
+      bj = -rep(0,nk[j])
       vj = vreg[j,]
       mj = sqrt(sum(vj^2))
       vj = vj / mj              # Standardize (divide by norm of vj)
-      a = poly.pval(y,Gj,uj,vj,sigma,bits)
+
+      limits.info = TG.limits(y, Aj, bj, vj, Sigma=diag(rep(sigma^2, n)))
+      a = TG.pvalue.base(limits.info, bits=bits)
       pv[j] = a$pv
       sxj = sx[vars[j]]
       vlo[j] = a$vlo * mj / sxj # Unstandardize (mult by norm of vj / sxj)
       vup[j] = a$vup * mj / sxj # Unstandardize (mult by norm of vj)
       vmat[j,] = vj * mj / sxj  # Unstandardize (mult by norm of vj / sxj)
 
-      a = poly.int(y,Gj,uj,vj,sigma,alpha,gridrange=gridrange,
-        flip=(sign[j]==-1),bits=bits)
+      a = TG.interval.base(limits.info,
+                           alpha=alpha,
+                           gridrange=gridrange,
+                           flip=(sign[j]==-1),
+                           bits=bits)
       ci[j,] = a$int * mj / sxj # Unstandardize (mult by norm of vj / sxj)
       tailarea[j,] = a$tailarea
 
@@ -427,18 +428,23 @@ larInf <- function(obj, sigma=NULL, alpha=0.1, k=NULL, type=c("active","all","ai
       vj = vj / mj             # Standardize (divide by norm of vj)
       sign[j] = sign(sum(vj*y))
       vj = sign[j] * vj
-      Gj = rbind(G,vj)
-      uj = c(u,0)
+      Aj = -rbind(G,vj)
+      bj = -c(u,0)
+
+      limits.info = TG.limits(y, Aj, bj, vj, Sigma=diag(rep(sigma^2, n)))
+      a = TG.pvalue.base(limits.info, bits=bits)
 
-      a = poly.pval(y,Gj,uj,vj,sigma,bits)
       pv[j] = a$pv
       sxj = sx[vars[j]]
       vlo[j] = a$vlo * mj / sxj # Unstandardize (mult by norm of vj / sxj)
       vup[j] = a$vup * mj / sxj # Unstandardize (mult by norm of vj / sxj)
       vmat[j,] = vj * mj / sxj  # Unstandardize (mult by norm of vj / sxj)
 
-      a = poly.int(y,Gj,uj,vj,sigma,alpha,gridrange=gridrange,
-        flip=(sign[j]==-1),bits=bits)
+      a = TG.interval.base(limits.info,
+                           alpha=alpha,
+                           gridrange=gridrange,
+                           flip=(sign[j]==-1),
+                           bits=bits)
       ci[j,] = a$int * mj / sxj # Unstandardize (mult by norm of vj / sxj)
       tailarea[j,] = a$tailarea
     }
